@@ -460,6 +460,7 @@ class Callable(param.Parameterized):
         self._is_overlay = False
         self.args = None
         self.kwargs = None
+        self._stream_memoization = self.memoize
 
     @property
     def argspec(self):
@@ -486,6 +487,7 @@ class Callable(param.Parameterized):
 
     def __call__(self, *args, **kwargs):
         # Nothing to do for callbacks that accept no arguments
+        kwarg_hash = kwargs.pop('memoization_hash', ())
         (self.args, self.kwargs) = (args, kwargs)
         if not args and not kwargs: return self.callable()
         inputs = [i for i in self.inputs if isinstance(i, DynamicMap)]
@@ -493,12 +495,12 @@ class Callable(param.Parameterized):
         for stream in [s for i in inputs for s in get_nested_streams(i)]:
             if stream not in streams: streams.append(stream)
 
-        memoize = self.memoize and not any(s.transient and s._triggering for s in streams)
-        values = tuple(tuple(sorted(s.contents.items())) for s in streams)
-        key = args + tuple(sorted(kwargs.items())) + values
+        memoize = self._stream_memoization and not any(s.transient and s._triggering for s in streams)
+        values = tuple(tuple(sorted(s.hashkey.items())) for s in streams)
+        key = args + kwarg_hash + values
 
-        hashed_key = util.deephash(key)
-        if memoize and hashed_key in self._memoized:
+        hashed_key = util.deephash(key) if self.memoize else None
+        if hashed_key is not None and memoize and hashed_key in self._memoized:
             return self._memoized[hashed_key]
 
         if self.argspec.varargs is not None:
@@ -591,14 +593,14 @@ def dynamicmap_memoization(callable_obj, streams):
     DynamicMap). Memoization is disabled if any of the streams require
     it it and are currently in a triggered state.
     """
-    memoization_state = bool(callable_obj.memoize)
-    callable_obj.memoize &= not any(s.transient and s._triggering for s in streams)
+    memoization_state = bool(callable_obj._stream_memoization)
+    callable_obj._stream_memoization &= not any(s.transient and s._triggering for s in streams)
     try:
         yield
     except:
         raise
     finally:
-        callable_obj.memoize = memoization_state
+        callable_obj._stream_memoization = memoization_state
 
 
 
@@ -782,12 +784,12 @@ class DynamicMap(HoloMap):
             low, high = util.max_range([kdim.range, kdim.soft_range])
             if low is not np.NaN:
                 if val < low:
-                    raise StopIteration("Key value %s below lower bound %s"
-                                        % (val, low))
+                    raise KeyError("Key value %s below lower bound %s"
+                                   % (val, low))
             if high is not np.NaN:
                 if val > high:
-                    raise StopIteration("Key value %s above upper bound %s"
-                                        % (val, high))
+                    raise KeyError("Key value %s above upper bound %s"
+                                   % (val, high))
 
     def event(self, **kwargs):
         """
@@ -837,6 +839,7 @@ class DynamicMap(HoloMap):
         # Additional validation needed to ensure kwargs don't clash
         kdims = [kdim.name for kdim in self.kdims]
         kwarg_items = [s.contents.items() for s in self.streams]
+        hash_items = tuple(tuple(sorted(s.hashkey.items())) for s in self.streams)+args
         flattened = [(k,v) for kws in kwarg_items for (k,v) in kws
                      if k not in kdims]
 
@@ -845,6 +848,8 @@ class DynamicMap(HoloMap):
             args = ()
         else:
             kwargs = dict(flattened)
+        if not isinstance(self.callback, Generator):
+            kwargs['memoization_hash'] = hash_items
 
         with dynamicmap_memoization(self.callback, self.streams):
             retval = self.callback(*args, **kwargs)
@@ -858,8 +863,11 @@ class DynamicMap(HoloMap):
         all ids are reset.
         """
         from ..util import Dynamic
-        return Dynamic(self, operation=lambda obj, **dynkwargs: obj.opts(options, **kwargs),
-                       streams=self.streams, shared_data=True, link_inputs=True)
+        dmap = Dynamic(self, operation=lambda obj, **dynkwargs: obj.opts(options, **kwargs),
+                       streams=self.streams, link_inputs=True)
+        dmap.data = OrderedDict([(k, v.opts(options, **kwargs))
+                                 for k, v in self.data.items()])
+        return dmap
 
 
     def clone(self, data=None, shared_data=True, new_type=None, link_inputs=True,
@@ -930,8 +938,10 @@ class DynamicMap(HoloMap):
 
         if data_slice:
             from ..util import Dynamic
-            return Dynamic(product, operation=lambda obj, **dynkwargs: obj[data_slice],
-                           streams=self.streams, shared_data=True)
+            dmap = Dynamic(self, operation=lambda obj, **dynkwargs: obj[data_slice],
+                           streams=self.streams)
+            dmap.data = product.data
+            return dmap
         return product
 
 
@@ -965,8 +975,10 @@ class DynamicMap(HoloMap):
                 if len(self):
                     slices = [slice(None) for _ in range(self.ndims)] + list(data_slice)
                     sliced = super(DynamicMap, sliced).__getitem__(tuple(slices))
-                return Dynamic(sliced, operation=lambda obj, **dynkwargs: obj[data_slice],
-                               streams=self.streams, shared_data=True)
+                dmap = Dynamic(self, operation=lambda obj, **dynkwargs: obj[data_slice],
+                               streams=self.streams)
+                dmap.data = sliced.data
+                return dmap
         return sliced
 
 
@@ -1046,8 +1058,10 @@ class DynamicMap(HoloMap):
             return dynamic_select(selection)
         else:
             from ..util import Dynamic
-            return Dynamic(selection, operation=dynamic_select,
-                           streams=self.streams, shared_data=True)
+            dmap = Dynamic(self, operation=dynamic_select, streams=self.streams)
+            dmap.data = selection.data
+            return dmap
+            
 
 
     def _cache(self, key, val):
@@ -1062,19 +1076,22 @@ class DynamicMap(HoloMap):
         self[key] = val
 
 
-    def map(self, map_fn, specs=None, clone=True):
+    def map(self, map_fn, specs=None, clone=True, link_inputs=True):
         """
         Recursively replaces elements using a map function when the
         specification applies. Extends regular map with functionality
-        to dynamically apply functions.
+        to dynamically apply functions. By default all streams are
+        still linked to the mapped object, to disable linked streams
+        set linked_inputs=False.
         """
         deep_mapped = super(DynamicMap, self).map(map_fn, specs, clone)
         if isinstance(deep_mapped, type(self)):
             from ..util import Dynamic
             def apply_map(obj, **dynkwargs):
                 return obj.map(map_fn, specs, clone)
-            dmap = Dynamic(deep_mapped, operation=apply_map,
-                           streams=self.streams, shared_data=True)
+            dmap = Dynamic(self, operation=apply_map, streams=self.streams,
+                           link_inputs=link_inputs)
+            dmap.data = deep_mapped.data
             return dmap
         return deep_mapped
 
@@ -1089,8 +1106,12 @@ class DynamicMap(HoloMap):
             from ..util import Dynamic
             def dynamic_relabel(obj, **dynkwargs):
                 return obj.relabel(group=group, label=label, depth=depth-1)
-            return Dynamic(relabelled, streams=self.streams, shared_data=True,
-                           operation=dynamic_relabel)
+            dmap = Dynamic(self, streams=self.streams, operation=dynamic_relabel)
+            dmap.data = relabelled.data
+            with util.disable_constant(dmap):
+                dmap.group = relabelled.group
+                dmap.label = relabelled.label
+            return dmap
         return relabelled
 
 
@@ -1338,8 +1359,8 @@ class GridSpace(UniformNdMapping):
 
     kdims = param.List(default=[Dimension("X"), Dimension("Y")], bounds=(1,2))
 
-    def __init__(self, initial_items=None, **params):
-        super(GridSpace, self).__init__(initial_items, **params)
+    def __init__(self, initial_items=None, kdims=None, **params):
+        super(GridSpace, self).__init__(initial_items, kdims=kdims, **params)
         if self.ndims > 2:
             raise Exception('Grids can have no more than two dimensions.')
 
