@@ -3,8 +3,10 @@ from ..core.data import Dataset, DataConversion
 from .annotation import * # noqa (API import)
 from .chart import * # noqa (API import)
 from .chart3d import * # noqa (API import)
+from .graphs import * # noqa (API import)
 from .path import * # noqa (API import)
 from .raster import * # noqa (API import)
+from .stats import * # noqa (API import)
 from .tabular import * # noqa (API import)
 
 
@@ -22,7 +24,6 @@ class ElementConversion(DataConversion):
         return self(BoxWhisker, kdims, vdims, groupby, **kwargs)
 
     def bivariate(self, kdims=None, vdims=None, groupby=None, **kwargs):
-        from ..interface.seaborn import Bivariate
         return self(Bivariate, kdims, vdims, groupby, **kwargs)
 
     def curve(self, kdims=None, vdims=None, groupby=None, **kwargs):
@@ -32,7 +33,6 @@ class ElementConversion(DataConversion):
         return self(ErrorBars, kdims, vdims, groupby, sort=True, **kwargs)
 
     def distribution(self, dim=None, groupby=[], **kwargs):
-        from ..interface.seaborn import Distribution
         if dim is None:
             if self._element.vdims:
                 dim = self._element.vdims[0]
@@ -44,7 +44,7 @@ class ElementConversion(DataConversion):
             return reindexed.groupby(groupby, HoloMap, Distribution, **kwargs)
         else:
             element = self._element
-            params = dict(vdims=[element.get_dimension(dim)],
+            params = dict(kdims=[element.get_dimension(dim)],
                           label=element.label)
             if element.group != element.params()['group'].default:
                 params['group'] = element.group
@@ -55,7 +55,7 @@ class ElementConversion(DataConversion):
         return self(HeatMap, kdims, vdims, groupby, **kwargs)
 
     def image(self, kdims=None, vdims=None, groupby=None, **kwargs):
-        return self(GridImage, kdims, vdims, groupby, **kwargs)
+        return self(Image, kdims, vdims, groupby, **kwargs)
 
     def points(self, kdims=None, vdims=None, groupby=None, **kwargs):
         return self(Points, kdims, vdims, groupby, **kwargs)
@@ -64,10 +64,6 @@ class ElementConversion(DataConversion):
         heatmap = self.heatmap(kdims, vdims, **kwargs)
         return Raster(heatmap.data, **dict(self._element.get_param_values(onlychanged=True)))
 
-    def regression(self, kdims=None, vdims=None, groupby=None, **kwargs):
-        from ..interface.seaborn import Regression
-        return self(Regression, kdims, vdims, groupby, **kwargs)
-
     def scatter(self, kdims=None, vdims=None, groupby=None, **kwargs):
         return self(Scatter, kdims, vdims, groupby, **kwargs)
 
@@ -95,7 +91,8 @@ Dataset._conversion_interface = ElementConversion
 
 
 def public(obj):
-    if not isinstance(obj, type): return False
+    if not isinstance(obj, type) or getattr(obj, 'abstract', False):
+        return False
     return issubclass(obj, Element)
 
 __all__ = list(set([_k for _k, _v in locals().items() if public(_v)]))
