@@ -14,20 +14,19 @@ import dask.dataframe as dd
 
 ds_version = LooseVersion(ds.__version__)
 
-from datashader.core import bypixel
-from datashader.pandas import pandas_pipeline
-from datashader.dask import dask_pipeline
-from datashape.dispatch import dispatch
-from datashape import discover as dsdiscover
+try:
+    from datashader.bundling import (directly_connect_edges as connect_edges,
+                                     hammer_bundle)
+except:
+    hammer_bundle, connect_edges = object, object
 
 from ..core import (Operation, Element, Dimension, NdOverlay,
                     CompositeOverlay, Dataset)
-from ..core.data import PandasInterface, DaskInterface, XArrayInterface
+from ..core.data import PandasInterface, XArrayInterface
 from ..core.sheetcoords import BoundingBox
-from ..core.util import get_param_values, basestring
-from ..element import Image, Path, Curve, Contours, RGB
+from ..core.util import get_param_values, basestring, datetime_types, dt_to_int
+from ..element import Image, Path, Curve, RGB, Graph
 from ..streams import RangeXY, PlotSize
-from ..plotting.util import fire
 
 
 class ResamplingOperation(Operation):
@@ -104,16 +103,47 @@ class ResamplingOperation(Operation):
                 else:
                     x0, x1 = self.p.x_range
                     ex0, ex1 = element.range(x)
-                    x_range = max([x0, ex0]), min([x1, ex1])
+                    x_range = np.max([x0, ex0]), np.min([x1, ex1])
+                if x_range[0] == x_range[1]:
+                    x_range = (x_range[0]-0.5, x_range[0]+0.5)
+                    
                 if self.p.expand or not self.p.y_range:
                     y_range = self.p.y_range or element.range(y)
                 else:
                     y0, y1 = self.p.y_range
                     ey0, ey1 = element.range(y)
-                    y_range = max([y0, ey0]), min([y1, ey1])
+                    y_range = np.max([y0, ey0]), np.min([y1, ey1])
             width, height = self.p.width, self.p.height
         (xstart, xend), (ystart, yend) = x_range, y_range
 
+        xtype = 'numeric'
+        if isinstance(xstart, datetime_types) or isinstance(xend, datetime_types):
+            xstart, xend = dt_to_int(xstart), dt_to_int(xend)
+            xtype = 'datetime'
+        elif not np.isfinite(xstart) and not np.isfinite(xend):
+            if element.get_dimension_type(x) in datetime_types:
+                xstart, xend = 0, 10000
+                xtype = 'datetime'
+            else:
+                xstart, xend = 0, 1
+        elif xstart == xend:
+            xstart, xend = (xstart-0.5, xend+0.5)
+        x_range = (xstart, xend)
+
+        ytype = 'numeric'
+        if isinstance(ystart, datetime_types) or isinstance(yend, datetime_types):
+            ystart, yend = dt_to_int(ystart), dt_to_int(yend)
+            ytype = 'datetime'
+        elif not np.isfinite(ystart) and not np.isfinite(yend):
+            if element.get_dimension_type(y) in datetime_types:
+                xstart, xend = 0, 10000
+                xtype = 'datetime'
+            else:
+                ystart, yend = 0, 1
+        elif ystart == yend:
+            ystart, yend = (ystart-0.5, yend+0.5)
+        y_range = (ystart, yend)
+
         # Compute highest allowed sampling density
         xspan = xend - xstart
         yspan = yend - ystart
@@ -124,7 +154,8 @@ class ResamplingOperation(Operation):
         xunit, yunit = float(xspan)/width, float(yspan)/height
         xs, ys = (np.linspace(xstart+xunit/2., xend-xunit/2., width),
                   np.linspace(ystart+yunit/2., yend-yunit/2., height))
-        return (x_range, y_range), (xs, ys), (width, height)
+
+        return (x_range, y_range), (xs, ys), (width, height), (xtype, ytype)
 
 
 
@@ -162,16 +193,15 @@ class aggregate(ResamplingOperation):
         xarray Dataset that can be aggregated.
         """
         paths = []
+        if isinstance(obj, Graph):
+            obj = obj.edgepaths
         kdims = list(obj.kdims)
         vdims = list(obj.vdims)
-        dims = obj.dimensions(label=True)[:2]
+        dims = obj.dimensions()[:2]
         if isinstance(obj, Path):
             glyph = 'line'
-            for p in obj.data:
-                df = pd.DataFrame(p, columns=obj.dimensions('key', True))
-                if isinstance(obj, Contours) and obj.vdims and obj.level:
-                    df[obj.vdims[0].name] = p.level
-                paths.append(df)
+            for p in obj.split(datatype='dataframe'):
+                paths.append(p)
         elif isinstance(obj, CompositeOverlay):
             element = None
             for key, el in obj.data.items():
@@ -213,11 +243,11 @@ class aggregate(ResamplingOperation):
         if category and df[category].dtype.name != 'category':
             df[category] = df[category].astype('category')
 
+        if any(df[d.name].dtype.kind == 'M' for d in (x, y)):
+            df = df.copy()
         for d in (x, y):
-            if df[d].dtype.kind == 'M':
-                param.main.warning('Casting %s dimension data to integer; '
-                                   'datashader cannot process datetime data', d)
-                df[d] = df[d].astype('int64') / 1000000.
+            if df[d.name].dtype.kind == 'M':
+                df[d.name] = df[d.name].astype('datetime64[ns]').astype('int64') * 10e-4
 
         return x, y, Dataset(df, kdims=kdims, vdims=vdims), glyph
 
@@ -234,7 +264,7 @@ class aggregate(ResamplingOperation):
         """
         # Compute overall bounds
         x, y = element.last.dimensions()[0:2]
-        (x_range, y_range), (xs, ys), (width, height) = self._get_sampling(element, x, y)
+        (x_range, y_range), (xs, ys), (width, height), (xtype, ytype) = self._get_sampling(element, x, y)
         agg_params = dict({k: v for k, v in self.p.items() if k in aggregate.params()},
                           x_range=x_range, y_range=y_range)
 
@@ -303,7 +333,7 @@ class aggregate(ResamplingOperation):
             return self._aggregate_ndoverlay(element, agg_fn)
 
         x, y, data, glyph = self.get_agg_data(element, category)
-        (x_range, y_range), (xs, ys), (width, height) = self._get_sampling(element, x, y)
+        (x_range, y_range), (xs, ys), (width, height), (xtype, ytype) = self._get_sampling(element, x, y)
 
         if x is None or y is None:
             xarray = xr.DataArray(np.full((height, width), np.NaN, dtype=np.float32),
@@ -320,13 +350,17 @@ class aggregate(ResamplingOperation):
             name = column
         vdims = [element.get_dimension(column)(name) if column
                  else Dimension('Count')]
-        params = dict(get_param_values(element), kdims=element.dimensions()[:2],
+        params = dict(get_param_values(element), kdims=[x, y],
                       datatype=['xarray'], vdims=vdims)
 
         dfdata = PandasInterface.as_dframe(data)
-        agg = getattr(cvs, glyph)(dfdata, x, y, self.p.aggregator)
+        agg = getattr(cvs, glyph)(dfdata, x.name, y.name, self.p.aggregator)
         if 'x_axis' in agg and 'y_axis' in agg:
             agg = agg.rename({'x_axis': x, 'y_axis': y})
+        if xtype == 'datetime':
+            agg[x.name] = agg[x.name].astype('datetime64[us]')
+        if ytype == 'datetime':
+            agg[y.name] = agg[y.name].astype('datetime64[us]')
 
         if agg.ndim == 2:
             # Replacing x and y coordinates to avoid numerical precision issues
@@ -342,6 +376,109 @@ class aggregate(ResamplingOperation):
 
 
 
+class regrid(ResamplingOperation):
+    """
+    regrid allows resampling a HoloViews Image type using specified
+    up- and downsampling functions defined using the aggregator and
+    interpolation parameters respectively. By default upsampling is
+    disabled to avoid unnecessarily upscaling an image that has to be
+    sent to the browser. Also disables expanding the image beyond its
+    original bounds avoiding unneccessarily padding the output array
+    with nan values.
+    """
+
+    aggregator = param.ObjectSelector(default='mean',
+        objects=['first', 'last', 'mean', 'mode', 'std', 'var', 'min', 'max'], doc="""
+        Aggregation method.
+        """)
+
+    expand = param.Boolean(default=False, doc="""
+       Whether the x_range and y_range should be allowed to expand
+       beyond the extent of the data.  Setting this value to True is
+       useful for the case where you want to ensure a certain size of
+       output grid, e.g. if you are doing masking or other arithmetic
+       on the grids.  A value of False ensures that the grid is only
+       just as large as it needs to be to contain the data, which will
+       be faster and use less memory if the resulting aggregate is
+       being overlaid on a much larger background.""")
+
+    interpolation = param.ObjectSelector(default='nearest',
+        objects=['linear', 'nearest'], doc="""
+        Interpolation method""")
+
+    upsample = param.Boolean(default=False, doc="""
+        Whether to allow upsampling if the source array is smaller
+        than the requested array. Setting this value to True will
+        enable upsampling using the interpolation method, when the
+        requested width and height are larger than what is available
+        on the source grid. If upsampling is disabled (the default)
+        the width and height are clipped to what is available on the
+        source array.""")
+
+    def _process(self, element, key=None):
+        if ds_version <= '0.5.0':
+            raise RuntimeError('regrid operation requires datashader>=0.6.0')
+
+        x, y = element.kdims
+        (x_range, y_range), _, (width, height), (xtype, ytype) = self._get_sampling(element, x, y)
+
+        coords = tuple(element.dimension_values(d, expanded=False)
+                       for d in [x, y])
+        coord_dict = {x.name: coords[0], y.name: coords[1]}
+        dims = [y.name, x.name]
+        arrays = []
+        for vd in element.vdims:
+            if element.interface is XArrayInterface:
+                xarr = element.data[vd.name]
+                if 'datetime' in (xtype, ytype):
+                    xarr = xarr.copy()
+                if dims != xarr.dims:
+                    xarr = xarr.transpose(*dims)
+            else:
+                arr = element.dimension_values(vd, flat=False)
+                xarr = xr.DataArray(arr, coords=coord_dict, dims=dims)
+            if xtype == "datetime":
+                xarr[x.name] = [dt_to_int(v) for v in xarr[x.name].values]
+            if ytype == "datetime":
+                xarr[y.name] = [dt_to_int(v) for v in xarr[y.name].values]
+            arrays.append(xarr)
+
+        # Disable upsampling if requested
+        (xstart, xend), (ystart, yend) = (x_range, y_range)
+        xspan, yspan = (xend-xstart), (yend-ystart)
+        if not self.p.upsample and self.p.target is None:
+            (x0, x1), (y0, y1) = element.range(0), element.range(1)
+            if isinstance(x0, datetime_types):
+                x0, x1 = dt_to_int(x0), dt_to_int(x1)
+            if isinstance(y0, datetime_types):
+                y0, y1 = dt_to_int(y0), dt_to_int(y1)
+            exspan, eyspan = (x1-x0), (y1-y0)
+            width = min([int((xspan/exspan) * len(coords[0])), width])
+            height = min([int((yspan/eyspan) * len(coords[1])), height])
+
+        # Get expanded or bounded ranges
+        cvs = ds.Canvas(plot_width=width, plot_height=height,
+                        x_range=x_range, y_range=y_range)
+        regridded = []
+        for xarr in arrays:
+            rarray = cvs.raster(xarr, upsample_method=self.p.interpolation,
+                                downsample_method=self.p.aggregator)
+            if xtype == "datetime":
+                rarray[x.name] = rarray[x.name].astype('datetime64[us]')
+            if ytype == "datetime":
+                rarray[y.name] = rarray[y.name].astype('datetime64[us]')
+            regridded.append(rarray)
+
+        regridded = xr.Dataset({vd.name: xarr for vd, xarr in zip(element.vdims, regridded)})
+        if xtype == 'datetime':
+            xstart, xend = np.array([xstart, xend]).astype('datetime64[us]')
+        if ytype == 'datetime':
+            ystart, yend = np.array([ystart, yend]).astype('datetime64[us]')  
+        bbox = BoundingBox(points=[(xstart, ystart), (xend, yend)])
+        return element.clone(regridded, bounds=bbox, datatype=['xarray'])
+
+
+
 class shade(Operation):
     """
     shade applies a normalization function followed by colormapping to
@@ -355,10 +492,13 @@ class shade(Operation):
     Iterable or a Callable.
     """
 
-    cmap = param.ClassSelector(default=fire, class_=(Iterable, Callable, dict), doc="""
-        Iterable or callable which returns colors as hex colors, to
-        be used for the colormap of single-layer datashader output. 
-        Callable type must allow mapping colors between 0 and 1.""")
+    cmap = param.ClassSelector(class_=(Iterable, Callable, dict), doc="""
+        Iterable or callable which returns colors as hex colors
+        or web color names (as defined by datashader), to be used
+        for the colormap of single-layer datashader output. 
+        Callable type must allow mapping colors between 0 and 1.
+        The default value of None reverts to Datashader's default
+        colormap.""")
 
     color_key = param.ClassSelector(class_=(Iterable, Callable, dict), doc="""
         Iterable or callable which returns colors as hex colors, to
@@ -385,6 +525,12 @@ class shade(Operation):
         Disable when you do not want the resulting plot to be interactive,
         e.g. when trying to display an interactive plot a second time.""")
 
+    min_alpha = param.Number(default=40, doc="""
+        The minimum alpha value to use for non-empty pixels when doing
+        colormapping, in [0, 255].  Use a higher value to avoid
+        undersaturation, i.e. poorly visible low-value datapoints, at
+        the expense of the overall dynamic range..""")
+
     @classmethod
     def concatenate(cls, overlay):
         """
@@ -432,7 +578,7 @@ class shade(Operation):
 
         # Compute shading options depending on whether
         # it is a categorical or regular aggregate
-        shade_opts = dict(how=self.p.normalization)
+        shade_opts = dict(how=self.p.normalization, min_alpha=self.p.min_alpha)
         if element.ndims > 2:
             kdims = element.kdims[1:]
             categories = array.shape[-1]
@@ -459,6 +605,10 @@ class shade(Operation):
         elif ds_version > '0.5.0' and self.p.normalization != 'eq_hist':
             shade_opts['span'] = element.range(vdim)
 
+        for d in kdims:
+            if array[d.name].dtype.kind == 'M':
+                array[d.name] = array[d.name].astype('datetime64[ns]').astype('int64') * 10e-4
+
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', r'invalid value encountered in true_divide')
             if np.isnan(array.data).all():
@@ -489,6 +639,62 @@ class datashade(aggregate, shade):
 
 
 
+class stack(Operation):
+    """
+    The stack operation allows compositing multiple RGB Elements using
+    the defined compositing operator.
+    """
+
+    compositor = param.ObjectSelector(objects=['add', 'over', 'saturate', 'source'],
+                                      default='over', doc="""
+        Defines how the compositing operation combines the images""")
+
+    def uint8_to_uint32(self, element):
+        img = np.dstack([element.dimension_values(d, flat=False)
+                         for d in element.vdims])
+        if img.shape[2] == 3: # alpha channel not included
+            alpha = np.ones(img.shape[:2])
+            if img.dtype.name == 'uint8':
+                alpha = (alpha*255).astype('uint8')
+            img = np.dstack([img, alpha])
+        if img.dtype.name != 'uint8':
+            img = (img*255).astype(np.uint8)
+        N, M, _ = img.shape
+        return img.view(dtype=np.uint32).reshape((N, M))
+
+    def _process(self, overlay, key=None):
+        if not isinstance(overlay, CompositeOverlay):
+            return overlay
+        elif len(overlay) == 1:
+            return overlay.last if isinstance(overlay, NdOverlay) else overlay.get(0)
+
+        imgs = []
+        for rgb in overlay:
+            if not isinstance(rgb, RGB):
+                raise TypeError('stack operation expect RGB type elements, '
+                                'not %s name.' % type(rgb).__name__)
+            rgb = rgb.rgb
+            dims = [kd.name for kd in rgb.kdims][::-1]
+            coords = {kd.name: rgb.dimension_values(kd, False)
+                      for kd in rgb.kdims}
+            imgs.append(tf.Image(self.uint8_to_uint32(rgb), coords=coords, dims=dims))
+
+        try:
+            imgs = xr.align(*imgs, join='exact')
+        except ValueError:
+            raise ValueError('RGB inputs to stack operation could not be aligned, '
+                             'ensure they share the same grid sampling.')
+
+        stacked = tf.stack(*imgs, how=self.p.compositor)
+        arr = shade.uint32_to_uint8(stacked.data)[::-1]
+        data = (coords[dims[1]], coords[dims[0]], arr[:, :, 0],
+                arr[:, :, 1], arr[:, :, 2])
+        if arr.shape[-1] == 4:
+            data = data + (arr[:, :, 3],)
+        return rgb.clone(data, datatype=[rgb.interface.datatype]+rgb.datatype)
+
+
+
 class dynspread(Operation):
     """
     Spreading expands each pixel in an Image based Element a certain
@@ -561,3 +767,54 @@ class dynspread(Operation):
         return element.clone(new_data)
 
 
+def split_dataframe(path_df):
+    """
+    Splits a dataframe of paths separated by NaNs into individual
+    dataframes.
+    """
+    splits = np.where(path_df.iloc[:, 0].isnull())[0]+1
+    return [df for df in np.split(path_df, splits) if len(df) > 1]
+
+
+class _connect_edges(Operation):
+
+    split = param.Boolean(default=False, doc="""
+        Determines whether bundled edges will be split into individual edges
+        or concatenated with NaN separators.""")
+
+    def _bundle(self, position_df, edges_df):
+        raise NotImplementedError('_connect edges is an abstract baseclass '
+                                  'and does not implement any actual bundling.')
+
+    def _process(self, element, key=None):
+        index = element.nodes.kdims[2].name
+        rename_edges = {d.name: v for d, v in zip(element.kdims[:2], ['source', 'target'])}
+        rename_nodes = {d.name: v for d, v in zip(element.nodes.kdims[:2], ['x', 'y'])}
+        position_df = element.nodes.redim(**rename_nodes).dframe([0, 1, 2]).set_index(index)
+        edges_df = element.redim(**rename_edges).dframe([0, 1])
+        paths = self._bundle(position_df, edges_df)
+        paths = paths.rename(columns={v: k for k, v in rename_nodes.items()})
+        paths = split_dataframe(paths) if self.p.split else [paths]
+        return element.clone((element.data, element.nodes, paths))
+
+
+class bundle_graph(_connect_edges, hammer_bundle):
+    """
+    Iteratively group edges and return as paths suitable for datashading.
+
+    Breaks each edge into a path with multiple line segments, and
+    iteratively curves this path to bundle edges into groups.
+    """
+
+    def _bundle(self, position_df, edges_df):
+        from datashader.bundling import hammer_bundle
+        return hammer_bundle.__call__(self, position_df, edges_df, **self.p)
+
+
+class directly_connect_edges(_connect_edges, connect_edges):
+    """
+    Given a Graph object will directly connect all nodes.
+    """
+
+    def _bundle(self, position_df, edges_df):
+        return connect_edges.__call__(self, position_df, edges_df)
